@@ -1,7 +1,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/afs/DARWIN/osi_vnodeops.c,v 1.4.2.6 2003/01/26 00:02:28 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/DARWIN/osi_vnodeops.c,v 1.5 2001/11/10 23:20:08 shadow Exp $");
 
 #include <afs/sysincludes.h>            /* Standard vendor system headers */
 #include <afs/afsincludes.h>            /* Afs-based standard headers */
@@ -158,7 +158,7 @@ struct vop_lookup_args /* {
     if (flags & ISDOTDOT) 
        VOP_UNLOCK(dvp, 0, p);
     AFS_GLOCK();
-    error = afs_lookup(VTOAFS(dvp), name, &vcp, cnp->cn_cred);
+    error = afs_lookup((struct vcache *)dvp, name, &vcp, cnp->cn_cred);
     AFS_GUNLOCK();
     if (error) {
         if (flags & ISDOTDOT) 
@@ -172,7 +172,7 @@ struct vop_lookup_args /* {
 	*ap->a_vpp = 0;
 	return (error);
     }
-    vp = AFSTOV(vcp);  /* always get a node if no error */
+    vp = (struct vnode *)vcp;  /* always get a node if no error */
 
     /* The parent directory comes in locked.  We unlock it on return
        unless the caller wants it left locked.
@@ -224,7 +224,7 @@ afs_vop_create(ap)
 
     /* vnode layer handles excl/nonexcl */
     AFS_GLOCK();
-    error = afs_create(VTOAFS(dvp), name, ap->a_vap, NONEXCL,
+    error = afs_create((struct vcache *)dvp, name, ap->a_vap, NONEXCL,
 	               ap->a_vap->va_mode, &vcp,
 	               cnp->cn_cred);
     AFS_GUNLOCK();
@@ -236,11 +236,11 @@ afs_vop_create(ap)
     }
 
     if (vcp) {
-	*ap->a_vpp = AFSTOV(vcp);
-	vn_lock(*ap->a_vpp, LK_EXCLUSIVE| LK_RETRY, p);
-        if (UBCINFOMISSING(*ap->a_vpp) ||
-            UBCINFORECLAIMED(*ap->a_vpp))
-                ubc_info_init(*ap->a_vpp);
+	*ap->a_vpp = (struct vnode *)vcp;
+	vn_lock((struct vnode *)vcp, LK_EXCLUSIVE| LK_RETRY, p);
+        if (UBCINFOMISSING((struct vnode *)vcp) ||
+            UBCINFORECLAIMED((struct vnode *)vcp))
+                ubc_info_init((struct vnode *)vcp);
     }
     else *ap->a_vpp = 0;
 
@@ -275,15 +275,15 @@ afs_vop_open(ap)
 	} */ *ap;
 {
     int error;
-    struct vcache *vc = VTOAFS(ap->a_vp);
+    struct vcache *vc = (struct vcache *)ap->a_vp;
     AFS_GLOCK();
     error = afs_open(&vc, ap->a_mode, ap->a_cred);
 #ifdef DIAGNOSTIC
-    if (AFSTOV(vc) != ap->a_vp)
+    if ((struct vnode *)vc != ap->a_vp)
 	panic("AFS open changed vnode!");
 #endif
     afs_BozonLock(&vc->pvnLock, vc);
-    osi_FlushPages(vc, ap->a_cred);
+    osi_FlushPages(vc);
     afs_BozonUnlock(&vc->pvnLock, vc);
     AFS_GUNLOCK();
     return error;
@@ -306,20 +306,9 @@ afs_vop_close(ap)
     else
         code=afs_close(avc, ap->a_fflag, &afs_osi_cred, ap->a_p);
     afs_BozonLock(&avc->pvnLock, avc);
-    osi_FlushPages(avc, ap->a_cred);        /* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(avc);        /* hold bozon lock, but not basic vnode lock */
     afs_BozonUnlock(&avc->pvnLock, avc);
     AFS_GUNLOCK();
-#ifdef AFS_DARWIN14_ENV
-    if (UBCINFOEXISTS(ap->a_vp) && ap->a_vp->v_ubcinfo->ui_refcount < 2) {
-      ubc_hold(ap->a_vp);
-      if (ap->a_vp->v_ubcinfo->ui_refcount < 2) {
-         printf("afs: Imminent ui_refcount panic\n");
-       } else {
-         printf("afs: WARNING: ui_refcount panic averted\n");
-       }
-    }
-#endif
-
     return code;
 }
 
@@ -334,7 +323,7 @@ afs_vop_access(ap)
 {
     int code;
     AFS_GLOCK();
-    code=afs_access(VTOAFS(ap->a_vp), ap->a_mode, ap->a_cred);
+    code=afs_access((struct vcache *)ap->a_vp, ap->a_mode, ap->a_cred);
     AFS_GUNLOCK();
     return code;
 }
@@ -349,7 +338,7 @@ afs_vop_getattr(ap)
 {
     int code;
     AFS_GLOCK();
-    code=afs_getattr(VTOAFS(ap->a_vp), ap->a_vap, ap->a_cred);
+    code=afs_getattr((struct vcache *)ap->a_vp, ap->a_vap, ap->a_cred);
     AFS_GUNLOCK();
     return code;
 }
@@ -364,7 +353,7 @@ afs_vop_setattr(ap)
 {
     int code;
     AFS_GLOCK();
-    code=afs_setattr(VTOAFS(ap->a_vp), ap->a_vap, ap->a_cred);
+    code=afs_setattr((struct vcache *)ap->a_vp, ap->a_vap, ap->a_cred);
     AFS_GUNLOCK();
     return code;
 }
@@ -378,10 +367,10 @@ afs_vop_read(ap)
 	} */ *ap;
 {
     int code;
-    struct vcache *avc=VTOAFS(ap->a_vp);
+    struct vcache *avc=(struct vcache *)ap->a_vp;
     AFS_GLOCK();
     afs_BozonLock(&avc->pvnLock, avc);
-    osi_FlushPages(avc, ap->a_cred);        /* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(avc);        /* hold bozon lock, but not basic vnode lock */
     code=afs_read(avc, ap->a_uio, ap->a_cred, 0, 0, 0);
     afs_BozonUnlock(&avc->pvnLock, avc);
     AFS_GUNLOCK();
@@ -413,7 +402,7 @@ afs_vop_pagein(ap)
     int nocommit = flags & UPL_NOCOMMIT;
 
     int code;
-    struct vcache *tvc=VTOAFS(vp);
+    struct vcache *tvc=(struct vcache *)vp;
 
     if (UBCINVALID(vp)) {
 #if DIAGNOSTIC
@@ -458,7 +447,7 @@ afs_vop_pagein(ap)
     aiov.iov_base = (caddr_t)ioaddr;
     AFS_GLOCK();
     afs_BozonLock(&tvc->pvnLock, tvc);
-    osi_FlushPages(tvc, ap->a_cred);        /* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(tvc);        /* hold bozon lock, but not basic vnode lock */
     code=afs_read(tvc, uio, cred, 0, 0, 0);
     if (code == 0) {
       ObtainWriteLock(&tvc->lock, 2);
@@ -467,11 +456,6 @@ afs_vop_pagein(ap)
     }
     afs_BozonUnlock(&tvc->pvnLock, tvc);
     AFS_GUNLOCK();
-
-    /* Zero out rest of last page if there wasn't enough data in the file */
-    if (code == 0 && auio.uio_resid > 0)
-	memset(aiov.iov_base, 0, auio.uio_resid);
-
     kernel_upl_unmap(kernel_map, pl);
     if (!nocommit) {
       if (code)
@@ -495,16 +479,16 @@ afs_vop_write(ap)
 	} */ *ap;
 {
     int code;
-    struct vcache *avc=VTOAFS(ap->a_vp);
+    struct vcache *avc=(struct vcache *)ap->a_vp;
     void *object;
     AFS_GLOCK();
     afs_BozonLock(&avc->pvnLock, avc);
-    osi_FlushPages(avc, ap->a_cred);        /* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(avc);        /* hold bozon lock, but not basic vnode lock */
     if (UBCINFOEXISTS(ap->a_vp))
        ubc_clean(ap->a_vp, 1);
     if (UBCINFOEXISTS(ap->a_vp))
        osi_VM_NukePages(ap->a_vp, ap->a_uio->uio_offset, ap->a_uio->uio_resid);
-    code=afs_write(VTOAFS(ap->a_vp), ap->a_uio, ap->a_ioflag, ap->a_cred, 0);
+    code=afs_write((struct vcache *)ap->a_vp, ap->a_uio, ap->a_ioflag, ap->a_cred, 0);
     afs_BozonUnlock(&avc->pvnLock, avc);
     AFS_GUNLOCK();
     return code;
@@ -536,7 +520,7 @@ afs_vop_pageout(ap)
     int nocommit = flags & UPL_NOCOMMIT;
 
     int code;
-    struct vcache *tvc=VTOAFS(vp);
+    struct vcache *tvc=(struct vcache *)vp;
 
     if (UBCINVALID(vp)) {
 #if DIAGNOSTIC
@@ -627,7 +611,7 @@ afs_vop_pageout(ap)
 
     AFS_GLOCK();
     afs_BozonLock(&tvc->pvnLock, tvc);
-    osi_FlushPages(tvc, ap->a_cred);        /* hold bozon lock, but not basic vnode lock */
+    osi_FlushPages(tvc);        /* hold bozon lock, but not basic vnode lock */
     ObtainWriteLock(&tvc->lock, 1);
     afs_FakeOpen(tvc);
     ReleaseWriteLock(&tvc->lock);
@@ -663,7 +647,7 @@ afs_vop_ioctl(ap)
 	        struct proc *a_p;
 	} */ *ap;
 {
-    struct vcache *tvc = VTOAFS(ap->a_vp);
+    struct vcache *tvc = (struct vcache *)ap->a_vp;
     struct afs_ioctl data;
     int error = 0;
   
@@ -733,9 +717,9 @@ afs_vop_fsync(ap)
     AFS_GLOCK();
     /*vflushbuf(vp, wait);*/
     if (ap->a_cred)
-      error=afs_fsync(VTOAFS(vp), ap->a_cred);
+      error=afs_fsync((struct vcache *)vp, ap->a_cred);
     else
-      error=afs_fsync(VTOAFS(vp), &afs_osi_cred);
+      error=afs_fsync((struct vcache *)vp, &afs_osi_cred);
     AFS_GUNLOCK();
     return error;
 }
@@ -768,7 +752,7 @@ afs_vop_remove(ap)
 
     GETNAME();
     AFS_GLOCK();
-    error =  afs_remove(VTOAFS(dvp), name, cnp->cn_cred);
+    error =  afs_remove((struct vcache *)dvp, name, cnp->cn_cred);
     AFS_GUNLOCK();
     cache_purge(vp);
     if (!error && UBCINFOEXISTS(vp)) {
@@ -825,7 +809,7 @@ afs_vop_link(ap)
 	goto out;
     }
     AFS_GLOCK();
-    error = afs_link(VTOAFS(vp), VTOAFS(dvp), name, cnp->cn_cred);
+    error = afs_link((struct vcache *)vp, (struct vcache *)dvp, name, cnp->cn_cred);
     AFS_GUNLOCK();
     FREE_ZONE(cnp->cn_pnbuf, cnp->cn_pnlen, M_NAMEI);
     if (dvp != vp)
@@ -917,7 +901,7 @@ abortit:
 
     AFS_GLOCK();
     /* XXX use "from" or "to" creds? NFS uses "to" creds */
-    error = afs_rename(VTOAFS(fdvp), fname, VTOAFS(tdvp), tname, tcnp->cn_cred);
+    error = afs_rename((struct vcache *)fdvp, fname, (struct vcache *)tdvp, tname, tcnp->cn_cred);
     AFS_GUNLOCK();
 
     VOP_UNLOCK(fvp, 0, p);
@@ -958,7 +942,7 @@ afs_vop_mkdir(ap)
 	panic("afs_vop_mkdir: no name");
 #endif
     AFS_GLOCK();
-    error = afs_mkdir(VTOAFS(dvp), name, vap, &vcp, cnp->cn_cred);
+    error = afs_mkdir((struct vcache *)dvp, name, vap, &vcp, cnp->cn_cred);
     AFS_GUNLOCK();
     if (error) {
 	VOP_ABORTOP(dvp, cnp);
@@ -967,8 +951,8 @@ afs_vop_mkdir(ap)
 	return(error);
     }
     if (vcp) {
-	*ap->a_vpp = AFSTOV(vcp);
-	vn_lock(*ap->a_vpp, LK_EXCLUSIVE|LK_RETRY, p);
+	*ap->a_vpp = (struct vnode *)vcp;
+	vn_lock((struct vnode *)vcp, LK_EXCLUSIVE|LK_RETRY, p);
     } else
 	*ap->a_vpp = 0;
     DROPNAME();
@@ -999,7 +983,7 @@ afs_vop_rmdir(ap)
     }
 
     AFS_GLOCK();
-    error = afs_rmdir(VTOAFS(dvp), name, cnp->cn_cred);
+    error = afs_rmdir((struct vcache *)dvp, name, cnp->cn_cred);
     AFS_GUNLOCK();
     DROPNAME();
     vput(dvp);
@@ -1023,7 +1007,7 @@ afs_vop_symlink(ap)
 
     GETNAME();
     AFS_GLOCK();
-    error = afs_symlink(VTOAFS(dvp), name, ap->a_vap, ap->a_target,
+    error = afs_symlink((struct vcache *)dvp, name, ap->a_vap, ap->a_target,
 	                cnp->cn_cred);
     AFS_GUNLOCK();
     DROPNAME();
@@ -1049,7 +1033,7 @@ afs_vop_readdir(ap)
 	   ap->a_ncookies); */
     off=ap->a_uio->uio_offset;
     AFS_GLOCK();
-    error= afs_readdir(VTOAFS(ap->a_vp), ap->a_uio, ap->a_cred,
+    error= afs_readdir((struct vcache *)ap->a_vp, ap->a_uio, ap->a_cred,
 	               ap->a_eofflag);
     AFS_GUNLOCK();
     if (!error && ap->a_ncookies != NULL) {
@@ -1095,7 +1079,7 @@ afs_vop_readlink(ap)
     int error;
 /*    printf("readlink %x\n", ap->a_vp);*/
     AFS_GLOCK();
-    error= afs_readlink(VTOAFS(ap->a_vp), ap->a_uio, ap->a_cred);
+    error= afs_readlink((struct vcache *)ap->a_vp, ap->a_uio, ap->a_cred);
     AFS_GUNLOCK();
     return error;
 }
@@ -1115,7 +1099,7 @@ afs_vop_inactive(ap)
 	vprint("afs_vop_inactive(): pushing active", vp);
 
     AFS_GLOCK();
-    afs_InactiveVCache(VTOAFS(vp), 0);   /* decrs ref counts */
+    afs_InactiveVCache((struct vcache *)vp, 0);   /* decrs ref counts */
     AFS_GUNLOCK();
     VOP_UNLOCK(vp, 0, ap->a_p);
     return 0;
@@ -1135,7 +1119,7 @@ afs_vop_reclaim(ap)
 
 #if 0 
     AFS_GLOCK();
-    error = afs_FlushVCache(VTOAFS(vp), &sl); /* tosses our stuff from vnode */
+    error = afs_FlushVCache((struct vcache *)vp, &sl); /* tosses our stuff from vnode */
     AFS_GUNLOCK();
     ubc_unlink(vp);
     if (!error && vp->v_data)
@@ -1160,7 +1144,7 @@ afs_vop_lock(ap)
 	} */ *ap;
 {
 	register struct vnode *vp = ap->a_vp;
-	register struct vcache *avc = VTOAFS(vp);
+	register struct vcache *avc = (struct vcache *)vp;
 
 	if (vp->v_tag == VT_NON)
 	        return (ENOENT);
@@ -1175,7 +1159,7 @@ afs_vop_unlock(ap)
 	} */ *ap;
 {
     struct vnode *vp = ap->a_vp;
-    struct vcache *avc = VTOAFS(vp);
+    struct vcache *avc = (struct vcache *)vp;
     return (lockmgr(&avc->rwlock, ap->a_flags | LK_RELEASE,
             &vp->v_interlock, ap->a_p));
 
@@ -1228,25 +1212,23 @@ afs_vop_print(ap)
 	} */ *ap;
 {
     register struct vnode *vp = ap->a_vp;
-    register struct vcache *vc = VTOAFS(ap->a_vp);
+    register struct vcache *vc = (struct vcache *)ap->a_vp;
     int s = vc->states;
+    char buf[20];
     printf("tag %d, fid: %ld.%x.%x.%x, opens %d, writers %d", vp->v_tag, vc->fid.Cell,
 	   vc->fid.Fid.Volume, vc->fid.Fid.Vnode, vc->fid.Fid.Unique, vc->opens,
 	   vc->execsOrWriters);
     printf("\n  states%s%s%s%s%s", (s&CStatd) ? " statd" : "", (s&CRO) ? " readonly" : "",(s&CDirty) ? " dirty" : "",(s&CMAPPED) ? " mapped" : "", (s&CVFlushed) ? " flush in progress" : "");
-    if (UBCISVALID(vp)) {
-        printf("\n  UBC: ");
-        if (UBCINFOEXISTS(vp)) {
-               printf("exists, ") ;
+    if (UBCISVALID(vp))
+        printf("\n  UBC: %s%s",
+               UBCINFOEXISTS(vp) ? "exists, " : "does not exist",
 #ifdef AFS_DARWIN14_ENV
-                 printf("refs %d%s%s", vp->v_ubcinfo->ui_refcount,
-                         ubc_issetflags(vp, UI_HASOBJREF) ? " HASOBJREF" : "",
-                         ubc_issetflags(vp, UI_WASMAPPED) ? " WASMAPPED" : "");
+               UBCINFOEXISTS(vp) ?
+                 sprintf(buf, "refs %d", vp->v_ubcinfo->ui_refcount),buf : "");
 #else
-                 printf("holdcnt %d", vp->v_ubcinfo->ui_holdcnt);
+               UBCINFOEXISTS(vp) ?
+                 sprintf(buf, "holdcnt %d", vp->v_ubcinfo->ui_holdcnt),buf : "");
 #endif
-        } else printf("does not exist");
-    }
     printf("\n");
     return 0;
 }
@@ -1257,7 +1239,7 @@ afs_vop_islocked(ap)
 	        struct vnode *a_vp;
 	} */ *ap;
 {
-    struct vcache *vc = VTOAFS(ap->a_vp);
+    struct vcache *vc = (struct vcache *)ap->a_vp;
     return lockstatus(&vc->rwlock);
 }
 
@@ -1317,7 +1299,7 @@ afs_vop_advlock(ap)
     cr=*p->p_cred->pc_ucred;
     pcred_unlock(p);
     AFS_GLOCK();
-    error= afs_lockctl(VTOAFS(ap->a_vp), ap->a_fl, ap->a_op, &cr,
+    error= afs_lockctl((struct vcache *)ap->a_vp, ap->a_fl, ap->a_op, &cr,
 	               (int) ap->a_id);
     AFS_GUNLOCK();
     return error;
