@@ -22,7 +22,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.14.2.11 2002/12/10 19:11:40 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_lookup.c,v 1.25 2001/11/21 16:01:31 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -54,7 +54,6 @@ extern struct inode_operations afs_symlink_iops, afs_dir_iops;
 
 afs_int32 afs_bulkStatsDone;
 static int bulkStatCounter = 0;	/* counter for bulk stat seq. numbers */
-int afs_fakestat_enable = 0;	/* 1: fakestat-all, 2: fakestat-crosscell */
 
 
 /* this would be faster if it did comparison as int32word, but would be 
@@ -91,9 +90,8 @@ char *afs_index(a, c)
 }
 
 /* call under write lock, evaluate mvid field from a mt pt.
- * avc is the vnode of the mount point object; must be write-locked.
- * advc is the vnode of the containing directory (optional; if NULL and
- *   EvalMountPoint succeeds, caller must initialize *avolpp->dotdot)
+ * avc is the vnode of the mount point object.
+ * advc is the vnode of the containing directory
  * avolpp is where we return a pointer to the volume named by the mount pt, if success
  * areq is the identity of the caller.
  *
@@ -110,7 +108,7 @@ EvalMountPoint(avc, advc, avolpp, areq)
     struct VenusFid tfid;
     struct cell *tcell;
     char   *cpos, *volnamep;
-    char   type, *buf;
+    char   type, buf[128];
     afs_int32  prefetchRO;          /* 1=>No  2=>Yes */
     afs_int32  mtptCell, assocCell, hac=0;
     afs_int32  samecell, roname, len;
@@ -179,18 +177,15 @@ EvalMountPoint(avc, advc, avolpp, areq)
      * Don't know why we do this. Would have still found it in above call - jpm.
      */
     if (!tvp && (prefetchRO == 2)) {
-	buf = (char *)osi_AllocSmallSpace(strlen(volnamep)+10);
+       strcpy(buf, volnamep);
+       afs_strcat(buf, ".readonly");
 
-	strcpy(buf, volnamep);
-	afs_strcat(buf, ".readonly");
+       tvp = afs_GetVolumeByName(buf, mtptCell, 1, areq, WRITE_LOCK);
 
-	tvp = afs_GetVolumeByName(buf, mtptCell, 1, areq, WRITE_LOCK);
-       
-	/* Try the associated linked cell if failed */
-	if (!tvp && hac && areq->volumeError) {
-	    tvp = afs_GetVolumeByName(buf, assocCell, 1, areq, WRITE_LOCK);
-	}
-	osi_FreeSmallSpace(buf);
+       /* Try the associated linked cell if failed */
+       if (!tvp && hac && areq->volumeError) {
+	  tvp = afs_GetVolumeByName(buf, assocCell, 1, areq, WRITE_LOCK);
+       }
     }
   
     if (!tvp) return ENODEV;       /* Couldn't find the volume */
@@ -198,7 +193,7 @@ EvalMountPoint(avc, advc, avolpp, areq)
     /* Don't cross mountpoint from a BK to a BK volume */
     if ((avc->states & CBackup) && (tvp->states & VBackup)) {
 	afs_PutVolume(tvp, WRITE_LOCK);
-	return ENODEV;
+	return ELOOP;
     }
 
     /* If we want (prefetched) the RO and it exists, then drop the
@@ -236,171 +231,11 @@ EvalMountPoint(avc, advc, avolpp, areq)
      * to the new path.
      */
     tvp->mtpoint = avc->fid;	/* setup back pointer to mtpoint */
-    if (advc) tvp->dotdot  = advc->fid;
+    tvp->dotdot  = advc->fid;
 
     *avolpp = tvp;
     return 0;
 }
-
-/*
- * afs_InitFakeStat
- *
- * Must be called on an afs_fakestat_state object before calling
- * afs_EvalFakeStat or afs_PutFakeStat.  Calling afs_PutFakeStat
- * without calling afs_EvalFakeStat is legal, as long as this
- * function is called.
- */
-
-void
-afs_InitFakeStat(state)
-    struct afs_fakestat_state *state;
-{
-    state->valid = 1;
-    state->did_eval = 0;
-    state->need_release = 0;
-}
-
-/*
- * afs_EvalFakeStat_int
- *
- * The actual implementation of afs_EvalFakeStat and afs_TryEvalFakeStat,
- * which is called by those wrapper functions.
- *
- * Only issues RPCs if canblock is non-zero.
- */
-int afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
-	struct vrequest *areq, int canblock)
-{
-    struct vcache *tvc, *root_vp;
-    struct volume *tvolp = NULL;
-    int code = 0;
-
-    osi_Assert(state->valid == 1);
-    osi_Assert(state->did_eval == 0);
-    state->did_eval = 1;
-    if (!afs_fakestat_enable)
-	return 0;
-    tvc = *avcp;
-    if (tvc->mvstat != 1)
-	return 0;
-
-    /* Is the call to VerifyVCache really necessary? */
-    code = afs_VerifyVCache(tvc, areq);
-    if (code)
-	goto done;
-    if (canblock) {
-	ObtainWriteLock(&tvc->lock, 599);
-	code = EvalMountPoint(tvc, NULL, &tvolp, areq);
-	ReleaseWriteLock(&tvc->lock);
-	if (code)
-	    goto done;
-	if (tvolp) {
-	    tvolp->dotdot = tvc->fid;
-	    tvolp->dotdot.Fid.Vnode = tvc->parentVnode;
-	    tvolp->dotdot.Fid.Unique = tvc->parentUnique;
-	}
-    }
-    if (tvc->mvid && (tvc->states & CMValid)) {
-	if (!canblock) {
-	    afs_int32 retry;
-
-	    do {
-		retry = 0;
-		ObtainWriteLock(&afs_xvcache, 597);
-		root_vp = afs_FindVCache(tvc->mvid, 0, 0, &retry, 0);
-		if (root_vp && retry) {
-		    ReleaseWriteLock(&afs_xvcache);
-		    afs_PutVCache(root_vp, 0);
-		}
-	    } while (root_vp && retry);
-	    ReleaseWriteLock(&afs_xvcache);
-	} else {
-	    root_vp = afs_GetVCache(tvc->mvid, areq, NULL, NULL, WRITE_LOCK);
-	}
-	if (!root_vp) {
-	    code = canblock ? ENOENT : 0;
-	    goto done;
-	}
-	if (tvolp) {
-	    /* Is this always kosher?  Perhaps we should instead use
-	     * NBObtainWriteLock to avoid potential deadlock.
-	     */
-	    ObtainWriteLock(&root_vp->lock, 598);
-	    if (!root_vp->mvid)
-		root_vp->mvid = osi_AllocSmallSpace(sizeof(struct VenusFid));
-	    *root_vp->mvid = tvolp->dotdot;
-	    ReleaseWriteLock(&root_vp->lock);
-	}
-	state->need_release = 1;
-	state->root_vp = root_vp;
-	*avcp = root_vp;
-	code = 0;
-    } else {
-	code = canblock ? ENOENT : 0;
-    }
-
-done:
-    if (tvolp)
-	afs_PutVolume(tvolp, WRITE_LOCK);
-    return code;
-}
-
-/*
- * afs_EvalFakeStat
- *
- * Automatically does the equivalent of EvalMountPoint for vcache entries
- * which are mount points.  Remembers enough state to properly release
- * the volume root vcache when afs_PutFakeStat() is called.
- *
- * State variable must be initialized by afs_InitFakeState() beforehand.
- *
- * Returns 0 when everything succeeds and *avcp points to the vcache entry
- * that should be used for the real vnode operation.  Returns non-zero if
- * something goes wrong and the error code should be returned to the user.
- */
-int
-afs_EvalFakeStat(avcp, state, areq)
-    struct vcache **avcp;
-    struct afs_fakestat_state *state;
-    struct vrequest *areq;
-{
-    return afs_EvalFakeStat_int(avcp, state, areq, 1);
-}
-
-/*
- * afs_TryEvalFakeStat
- *
- * Same as afs_EvalFakeStat, but tries not to talk to remote servers
- * and only evaluate the mount point if all the data is already in
- * local caches.
- *
- * Returns 0 if everything succeeds and *avcp points to a valid
- * vcache entry (possibly evaluated).
- */
-int
-afs_TryEvalFakeStat(avcp, state, areq)
-    struct vcache **avcp;
-    struct afs_fakestat_state *state;
-    struct vrequest *areq;
-{
-    return afs_EvalFakeStat_int(avcp, state, areq, 0);
-}
-
-/*
- * afs_PutFakeStat
- *
- * Perform any necessary cleanup at the end of a vnode op, given that
- * afs_InitFakeStat was previously called with this state.
- */
-void
-afs_PutFakeStat(state)
-    struct afs_fakestat_state *state;
-{
-    osi_Assert(state->valid == 1);
-    if (state->need_release)
-	afs_PutVCache(state->root_vp, 0);
-    state->valid = 0;
-}
     
 afs_ENameOK(aname)
     register char *aname; {
@@ -529,7 +364,7 @@ int afs_DoBulkStat(adp, dirCookie, areqp)
     struct dcache *dcp;		/* chunk containing the dir block */
     char *statMemp;		/* status memory block */
     char *cbfMemp;		/* callback and fid memory block */
-    long temp;			/* temp for holding chunk length, &c. */
+    afs_size_t temp;		/* temp for holding chunk length, &c. */
     struct AFSFid *fidsp;	/* file IDs were collecting */
     struct AFSCallBack *cbsp;	/* call back pointers */
     struct AFSCallBack *tcbp;	/* temp callback ptr */
@@ -550,7 +385,7 @@ int afs_DoBulkStat(adp, dirCookie, areqp)
     long startTime;		/* time we started the call,
 				 * for callback expiration base
 				 */
-    int statSeqNo;		/* Valued of file size to detect races */
+    afs_size_t statSeqNo;	/* Valued of file size to detect races */
     int code;			/* error code */
     long newIndex;		/* new index in the dir */
     struct DirEntry *dirEntryp;	/* dir entry we are examining */
@@ -602,7 +437,7 @@ tagain:
     code = afs_VerifyVCache(adp, areqp);
     if (code) goto done;
 
-    dcp = afs_GetDCache(adp, 0, areqp, &temp, &temp, 1);
+    dcp = afs_GetDCache(adp, (afs_size_t) 0, areqp, &temp, &temp, 1);
     if (!dcp) {
 	code = ENOENT;
 	goto done;
@@ -610,6 +445,7 @@ tagain:
 
     /* lock the directory cache entry */
     ObtainReadLock(&adp->lock);
+    ObtainReadLock(&dcp->lock);
 
     /*
      * Make sure that the data in the cache is current. There are two
@@ -618,15 +454,22 @@ tagain:
      * 2. The cache data is no longer valid
      */
     while ((adp->states & CStatd)
-	   && (dcp->flags & DFFetching)
+	   && (dcp->dflags & DFFetching)
 	   && hsame(adp->m.DataVersion, dcp->f.versionNo)) {
-	dcp->flags |= DFWaiting;
+	afs_Trace4(afs_iclSetp, CM_TRACE_DCACHEWAIT,
+			ICL_TYPE_STRING, __FILE__,
+			ICL_TYPE_INT32, __LINE__,
+			ICL_TYPE_POINTER, dcp,
+			ICL_TYPE_INT32, dcp->dflags);
+	ReleaseReadLock(&dcp->lock);
 	ReleaseReadLock(&adp->lock);
 	afs_osi_Sleep(&dcp->validPos);
 	ObtainReadLock(&adp->lock);
+	ObtainReadLock(&dcp->lock);
     }
     if (!(adp->states & CStatd)
 	|| !hsame(adp->m.DataVersion, dcp->f.versionNo)) {
+	ReleaseReadLock(&dcp->lock);
 	ReleaseReadLock(&adp->lock);
 	afs_PutDCache(dcp);
 	goto tagain;
@@ -746,6 +589,7 @@ tagain:
     }	/* while loop over all dir entries */
 
     /* now release the dir lock and prepare to make the bulk RPC */
+    ReleaseReadLock(&dcp->lock);
     ReleaseReadLock(&adp->lock);
 
     /* release the chunk */
@@ -773,20 +617,22 @@ tagain:
 #ifdef RX_ENABLE_LOCKS
 	    AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
-#ifdef notdef
-	    code = RXAFS_InlineBulkStatus(tcp->id, &fidParm, &statParm,
-					  &cbParm, &volSync);
-	    if (code == RXGEN_OPCODE) {
-#endif
+
+	    if (!(tcp->srvr->server->flags & SNO_INLINEBULK)) {
+		code = RXAFS_InlineBulkStatus(tcp->id, &fidParm, &statParm,
+					      &cbParm, &volSync);
+		if (code == RXGEN_OPCODE) {
+		    tcp->srvr->server->flags |= SNO_INLINEBULK;
+		    inlinebulk = 0;
+		    code = RXAFS_BulkStatus(tcp->id, &fidParm, &statParm, 
+					    &cbParm, &volSync);
+		} else
+		    inlinebulk=1;
+	    } else {
+		inlinebulk=0;
 		code = RXAFS_BulkStatus(tcp->id, &fidParm, &statParm, &cbParm,
 					&volSync);
-		inlinebulk=0;
-#ifdef notdef
-	    } else {
-		inlinebulk=1;
 	    }
-#endif
-
 #ifdef RX_ENABLE_LOCKS
 	    AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
@@ -1086,44 +932,20 @@ afs_lookup(adp, aname, avcp, acred)
     int no_read_access = 0;
     struct sysname_info sysState;   /* used only for @sys checking */
     int dynrootRetry = 1;
-    struct afs_fakestat_state fakestate;
-    int tryEvalOnly = 0;
 
     AFS_STATCNT(afs_lookup);
-    afs_InitFakeStat(&fakestate);
-
-    if (code = afs_InitReq(&treq, acred))
-	goto done;
-
 #ifdef	AFS_OSF_ENV
-    ndp->ni_dvp = AFSTOV(adp);
+    ndp->ni_dvp = (struct vnode *)adp;
     memcpy(aname, ndp->ni_ptr, ndp->ni_namelen);
     aname[ndp->ni_namelen] = '\0';
 #endif	/* AFS_OSF_ENV */
 
-#if defined(AFS_DARWIN_ENV)
-    /* Workaround for MacOSX Finder, which tries to look for
-     * .DS_Store and Contents under every directory.
-     */
-    if (afs_fakestat_enable && adp->mvstat == 1) {
-	if (strcmp(aname, ".DS_Store") == 0)
-	    tryEvalOnly = 1;
-	if (strcmp(aname, "Contents") == 0)
-	    tryEvalOnly = 1;
-    }
-#endif
-
-    if (tryEvalOnly)
-	code = afs_TryEvalFakeStat(&adp, &fakestate, &treq);
-    else
-	code = afs_EvalFakeStat(&adp, &fakestate, &treq);
-    if (tryEvalOnly && adp->mvstat == 1)
-	code = ENOENT;
-    if (code)
-	goto done;
-
     *avcp = (struct vcache *) 0;   /* Since some callers don't initialize it */
 
+    if (code = afs_InitReq(&treq, acred)) { 
+	goto done;
+    }
+
     /* come back to here if we encounter a non-existent object in a read-only
        volume's directory */
 
@@ -1145,7 +967,7 @@ afs_lookup(adp, aname, avcp, acred)
 #ifdef	AFS_OSF_ENV
 	    extern struct vcache *afs_globalVp;
 	    if (adp == afs_globalVp) {
-		struct vnode *rvp = AFSTOV(adp);
+		struct vnode *rvp = (struct vnode *)adp;
 /*
 		ndp->ni_vp = rvp->v_vfsp->vfs_vnodecovered;
 		ndp->ni_dvp = ndp->ni_vp;
@@ -1188,7 +1010,7 @@ afs_lookup(adp, aname, avcp, acred)
 
     /* Check for read access as well.  We need read access in order to
        stat files, but not to stat subdirectories. */
-    if (!afs_AccessOK(adp, PRSFS_LOOKUP, &treq, CHECK_MODE_BITS))
+    if (!afs_AccessOK(adp, PRSFS_READ, &treq, CHECK_MODE_BITS))
 	no_read_access = 1;
 
     /* special case lookup of ".".  Can we check for it sooner in this code,
@@ -1248,12 +1070,12 @@ afs_lookup(adp, aname, avcp, acred)
 
     {
     register struct dcache *tdc;
-    afs_int32 dirOffset, dirLen;
+    afs_size_t dirOffset, dirLen;
     ino_t theDir;
     struct VenusFid tfid;
 
     /* now we have to lookup the next fid */
-    tdc = afs_GetDCache(adp, 0, &treq, &dirOffset, &dirLen, 1);
+    tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &dirOffset, &dirLen, 1);
     if (!tdc) {
       *avcp = (struct vcache *)0;  /* redundant, but harmless */
       code = EIO;
@@ -1263,6 +1085,7 @@ afs_lookup(adp, aname, avcp, acred)
     /* now we will just call dir package with appropriate inode.
       Dirs are always fetched in their entirety for now */
     ObtainReadLock(&adp->lock);
+    ObtainReadLock(&tdc->lock);
 
     /*
      * Make sure that the data in the cache is current. There are two
@@ -1271,15 +1094,17 @@ afs_lookup(adp, aname, avcp, acred)
      * 2. The cache data is no longer valid
      */
     while ((adp->states & CStatd)
-	   && (tdc->flags & DFFetching)
+	   && (tdc->dflags & DFFetching)
 	   && hsame(adp->m.DataVersion, tdc->f.versionNo)) {
-	tdc->flags |= DFWaiting;
+	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&adp->lock);
 	afs_osi_Sleep(&tdc->validPos);
 	ObtainReadLock(&adp->lock);
+	ObtainReadLock(&tdc->lock);
     }
     if (!(adp->states & CStatd)
 	|| !hsame(adp->m.DataVersion, tdc->f.versionNo)) {
+	ReleaseReadLock(&tdc->lock);
 	ReleaseReadLock(&adp->lock);
 	afs_PutDCache(tdc);
 	goto redo;
@@ -1310,6 +1135,7 @@ afs_lookup(adp, aname, avcp, acred)
     }
     tname = sysState.name;
 
+    ReleaseReadLock(&tdc->lock);
     afs_PutDCache(tdc);
 
     if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry) {
@@ -1396,36 +1222,19 @@ afs_lookup(adp, aname, avcp, acred)
     } /* sub-block just to reduce stack usage */
 
     if (tvc) {
-	int force_eval = afs_fakestat_enable ? 0 : 1;
-
-	if (adp->states & CForeign)
+       if (adp->states & CForeign)
 	   tvc->states |= CForeign;
 	tvc->parentVnode = adp->fid.Fid.Vnode;
 	tvc->parentUnique = adp->fid.Fid.Unique;
 	tvc->states &= ~CBulkStat;
 
-	if (afs_fakestat_enable == 2 && tvc->mvstat == 1) {
-	    ObtainSharedLock(&tvc->lock, 680);
-	    if (!tvc->linkData) {
-		UpgradeSToWLock(&tvc->lock, 681);
-		code = afs_HandleLink(tvc, &treq);
-		ConvertWToRLock(&tvc->lock);
-	    } else {
-		ConvertSToRLock(&tvc->lock);
-		code = 0;
-	    }
-	    if (!code && !afs_strchr(tvc->linkData, ':'))
-		force_eval = 1;
-	    ReleaseReadLock(&tvc->lock);
-	}
-
 #if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
         if (!(flags & AFS_LOOKUP_NOEVAL))
           /* don't eval mount points */
 #endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
-	if (tvc->mvstat == 1 && force_eval) {
-	    /* a mt point, possibly unevaluated */
-	    struct volume *tvolp;
+	if (tvc->mvstat == 1) {
+	  /* a mt point, possibly unevaluated */
+	  struct volume *tvolp;
 
 	    ObtainWriteLock(&tvc->lock,133);
 	    code = EvalMountPoint(tvc, adp, &tvolp, &treq);
@@ -1525,7 +1334,6 @@ done:
 	    if (!FidCmp(&(tvc->fid), &(adp->fid))) { 
 		afs_PutVCache(*avcp, WRITE_LOCK);
 		*avcp = NULL;
-		afs_PutFakeStat(&fakestate);
 		return afs_CheckCode(EISDIR, &treq, 18);
 	    }
 	}
@@ -1549,7 +1357,6 @@ done:
 	    /* So Linux inode cache is up to date. */
 	    code = afs_VerifyVCache(tvc, &treq);
 #else
-	    afs_PutFakeStat(&fakestate);
 	    return 0;  /* can't have been any errors if hit and !code */
 #endif
 	}
@@ -1563,6 +1370,5 @@ done:
        *avcp = (struct vcache *)0;
     }
 
-    afs_PutFakeStat(&fakestate);
     return code;
 }
