@@ -10,14 +10,14 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/rx/rx_trace.c,v 1.7 2001/07/12 19:58:56 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/rx/rx_trace.c,v 1.10 2001/10/09 23:59:46 shadow Exp $");
 
 #ifdef RXDEBUG
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#else
 #ifdef HAVE_STRING_H
 #include <string.h>
+#else
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
 #endif
 #endif
 #ifdef AFS_NT40_ENV
@@ -36,7 +36,7 @@ char rxi_tracename[80]="/tmp/rxcalltrace";
 #else
 char rxi_tracename[80]="\0Change This pathname (and preceding NUL) to initiate tracing";
 #endif
-int rxi_logfd = 0;
+int rxi_logfd = -1;
 char rxi_tracebuf[4096];
 afs_uint32 rxi_tracepos = 0;
 
@@ -52,7 +52,7 @@ struct rx_trace {
 
 void rxi_flushtrace()
 {
-    if (rxi_logfd)
+    if (rxi_logfd >= 0)
 	write(rxi_logfd, rxi_tracebuf, rxi_tracepos);
     rxi_tracepos = 0;
 }
@@ -67,9 +67,9 @@ void rxi_calltrace(event, call)
   if (!rxi_tracename[0])
     return;
 
-  if (!rxi_logfd) {
+  if (rxi_logfd < 0) {
     rxi_logfd = open(rxi_tracename, O_WRONLY | O_CREAT | O_TRUNC, 0777);
-    if (!rxi_logfd)
+    if (rxi_logfd < 0)
       rxi_tracename[0] = '\0';
   }
   clock_GetTime(&now);
@@ -119,7 +119,7 @@ void rxi_calltrace(event, call)
 	 break;
     }
 
-  bcopy(&rxtinfo, rxi_tracebuf+rxi_tracepos, sizeof(struct rx_trace));
+  memcpy(rxi_tracebuf+rxi_tracepos, &rxtinfo, sizeof(struct rx_trace));
   rxi_tracepos += sizeof(struct rx_trace);
   if (rxi_tracepos >= (4096 - sizeof(struct rx_trace)))
     rxi_flushtrace();
@@ -153,7 +153,7 @@ char **argv;
   }
 
   rxi_logfd = open(rxi_tracename, O_RDONLY);
-  if (!rxi_logfd) {
+  if (rxi_logfd < 0) {
     perror("");
     exit(errno);
   }
